/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : DString.c
* 		Shared dynamicly allocating string functions. These are meant 
*		to replace existing str* functions with ones that are buffer
*		and NULL safe.
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*		12/05/2003 - strcmp and strcasecmp added.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "../Commoninclude.h"


/******************************************************************************
*	dstrcmp(char *, char *)
*		Null safe version of strcmp function
*
*	Args :
*		char * string1		String to compare.
*		char * string2		String to compare.
*
*       Updated :
******************************************************************************/
int dstrcmp(char * string1, char * string2)
{
	int Status = -1;
	
	if(string1 && string2)
	{
		Status = strcmp(string1, string2);
	}
	
	return(Status);
}

/******************************************************************************
*	dstrcasecmp(char *, char *)
*		Null safe version of strcmp function
*
*	Args :
*		char * string1		String to compare.
*		char * string2		String to compare.
*
*       Updated :
******************************************************************************/
int dstrcasecmp(char * string1, char * string2)
{
	int Status = -1;
	
	if(string1 && string2)
	{
		Status = strcasecmp(string1, string2);
	}
	
	return(Status);
}

/******************************************************************************
*       char * dstradvance(char * instr, unsigned advance)
*		Advances the instr a specified amount of positions.
*
*	Args :
*		char * instr		String point to operate on.
*		unsigned advance	Number of positions to advance.
*
*       Updated :
******************************************************************************/
char * dstradvance(char * instr, unsigned advance)
{
	unsigned i;
	
	if(instr)
	{
		for(i=0;i<advance;i++)
		{
			*instr++;
			if(instr == 0)
			{
				break;
			}
		}
	}
	
	return(instr);
}

/******************************************************************************
*       char * dstrstr(char * instr, char * findstr)
*		Safe version of strstr.
*
*	Args :
*		char * instr		Hay stack.
*		char * findstr		Needle.
*
*       Updated :
******************************************************************************/	
char * dstrstr(char * instr, char * findstr)
{
	if(instr && findstr)
	{
		return(strstr(instr, findstr));
	} else {
		return(NULL);
	}
}

/******************************************************************************
*       char * dstrcat(char * instr, char * addstr)
*		Safe version of strcat.
*
*	Args :
*		char * instr		String that wil be add too.
*		char * addstr		String to add.
*
*       Updated :
******************************************************************************/
char * dstrcat(char * instr, char * addstr)
{
	if(addstr)
	{
		instr = drealloc(instr, dstrlen(instr) + dstrlen(addstr) + 1);
		strcat(instr, addstr);
	}
	
	return(instr);
}

/******************************************************************************
*       char * dstrcpy(char * instr, char * newstr)
*		Safe version of strcpy.
*
*	Args :
*		char * instr		Destenation string to copy too.
*		char * newstr		Source string to copy from.
*
*       Updated :
******************************************************************************/
char * dstrcpy(char * instr, char * newstr)
{
	if(newstr)
	{
		instr = drealloc(instr, dstrlen(newstr) + 1);
		strcpy(instr, newstr);
	}
	return(instr);
}

/******************************************************************************
*       char * dstrncpy(char * instr, char * newstr, int len)
*		Safe version of strncpy.
*
*	Args :
*               char * instr            Destenation string to copy too.
*               char * newstr           Source string to copy from.
*		int len			Number of bytes to copy.
*
*       Updated :
******************************************************************************/
char * dstrncpy(char * instr, char * newstr, int len)
{
	if(newstr && len)
	{
		instr = drealloc(instr, len);
		strncpy(instr, newstr, len);
		instr[len] = 0;
	}
	return(instr);
}

/******************************************************************************
*       char * dfree(char * instr)
*		Safe version of free.
*
*	Args :
*		char * instr	Pointer to free.
*
*       Updated :
******************************************************************************/
char * dfree(char * instr)
{
	if(instr)
	{
		free(instr);
	} else {
		//printf("[af]-[NULL]\n");
	}
	return(NULL);
}

/******************************************************************************
*       char * drealloc(char * instr, unsigned size)
*		Safe version of malloc/realloc. bzero's all allocated strings.
*
*	Args :
*		char * instr	Pointer to allocate to.
*		unsigned size	Amount to allocate.
*
*       Updated :
******************************************************************************/
char * drealloc(char * instr, unsigned size)
{
	unsigned NewString = 0;
	
	if(size)
	{
		if(!instr)
		{
			NewString = 1;
		}
		
		instr = (char *) realloc(instr, size);
	
		if(NewString)
		{
			memset(instr, 0, size);
		}
	} else {
		if(instr)
		{
			instr = dfree(instr);
		}
	}
	return(instr);
}

/******************************************************************************
*       size_t dstrlen(char * instr)
*		Safe version of strlen.
*
*	Args :
*		char * instr	String to measure.
*
*       Updated :
******************************************************************************/
size_t dstrlen(char * instr)
{
	if(instr)
	{
		return(strlen(instr));
	} else {
		return(0);
	}
}

/******************************************************************************
*       char * dfiletostring(FILE *fp)
*		Reads the contents of passed handle and outputs its contents as
*		a string. Will stop at EOL.
*
*	Args :
*		FILE *fp	Handle to read from.
*
*       Updated :
******************************************************************************/
char * dfiletostring(FILE *fp)
{
	char * OutString = 0;
	char * ReadBuffer = 0;
	int len;
	
	ReadBuffer = drealloc(ReadBuffer, 1024);
	
	if(fp)
	{
		do
		{
			len = fread(ReadBuffer, 1, 1023, fp);
			if(len)
			{
				ReadBuffer[len] = 0;
				OutString = dstrcat(OutString, ReadBuffer);
			}
		}while(!feof(fp));
	}
	
	ReadBuffer = dfree(ReadBuffer);
	
	return(OutString);
}

/******************************************************************************
*       char * dstrtoupper(char * instr)
*		Converts string to all UPPER case.
*
*	Args :
*		char * instr	String to convert
*
*       Updated :
******************************************************************************/
char * dstrtoupper(char * instr)
{
	unsigned int i;
	
	if(instr)
	{
		for(i=0;i<dstrlen(instr);i++)
		{
			instr[i] = toupper(instr[i]);
		}
	}
	return(instr);
}

/******************************************************************************
*       char * dstrtolower(char * instr)
*		Converts string to all lower case.
*
*	Args :
*		char * instr	String to convert
*
*       Updated :
******************************************************************************/
char * dstrtolower(char * instr)
{
	unsigned int i;
	
	if(instr)
	{
		for(i=0;i<dstrlen(instr);i++)
		{
			instr[i] = tolower(instr[i]);
		}
	}
	return(instr);
}

/******************************************************************************
*       char * dstrclean(char * instr)
*		Removes leading a trailing whitespaces from string.
*
*	Args :
*		char * instr 	String to be cleaned.
*
*       Updated :
******************************************************************************/
char * dstrclean(char * instr)
{

	char WhiteChars[] = "  \t\r\n";
	unsigned i = 0;
	char * OutStr = 0;
	char * LocalInstr = 0;
	
	if(instr)
	{
		while(instr[i])
		{
			if(!strchr(WhiteChars, instr[i]))
			{
				LocalInstr = dstradvance(instr, i);
				OutStr = dstrcpy(OutStr, LocalInstr);
				instr = dstrcpy(instr, OutStr);
				OutStr = dfree(OutStr);
				break;
			}
			i++;
		}
		
		for(i=dstrlen(instr);i>0;i--)
		{
			if(!strchr(WhiteChars, instr[i]))
			{
				instr[i+1] = 0;
				break;
			}
		}
	}
	return(instr);
}

/******************************************************************************
*       char * dsubstrreplace(char * hay, char * needle, char * replace)
*		Replaces substrings within a string with another string.
*
*	Args :
*		char * hay		Haystack to search.
*		char * needle		Needle to find.
*		char * replace		String to replace with.
*
*       Updated :
******************************************************************************/
char * dsubstrreplace(char * hay, char * needle, char * replace)
{
	char * p = 0;
	char * pe = 0;
	char * tSegment = 0;
	int NeedleLen = 0;
	int i;
	unsigned Len = 0;
	unsigned FindCount =0 ;
			
	if(hay && needle)
	{
		NeedleLen = dstrlen(needle);

		p = hay;
		while(p)
		{
			p = strstr(hay, needle);
			if(p)
			{
				FindCount++;
				pe = p;
				for(i=0;i<NeedleLen;i++)
				{
					*pe++;
				}
				tSegment = dstrcpy(tSegment, pe);
				Len = dstrlen(hay) - dstrlen(tSegment) - NeedleLen;
				hay[Len] = 0;

				hay = drealloc(hay, dstrlen(hay) + dstrlen(tSegment) + dstrlen(replace) + 2);
				hay = dstrcat(hay, replace);
				hay = dstrcat(hay, tSegment);
				tSegment = dfree(tSegment);
			}
		}
	}
	return(hay);
}

/******************************************************************************
*       unsigned dcountsubstring(char * hay, char * needle)
*		Counts the number of occurances of a substring in a string.
*
*	Args :
*		char * hay	Haystack to search.
*		char * needle	Needle to count.
*
*       Updated :
******************************************************************************/
unsigned dcountsubstring(char * hay, char * needle)
{
	char *p = 0;
	unsigned Count = 0;
	
	if(hay && needle)
	{
		p = hay;
		while(p)
		{
			p = strstr(p, needle);
			if(p)
			{
				Count++;
			}
		}
	}

	return(Count);
}

/******************************************************************************
*       char * dsprintf(char *instr, char *format, ...)
*		Safe version of sprintf.
*	
*	Args :
*		char *instr	Pointer to store output in.
*		char *format	Formating string.
*		...		Args.
*
*       Updated :
******************************************************************************/			
char * dsprintf(char *instr, char *format, ...)
{
	unsigned Len = 0;
	va_list arg_list;
	
	if(format)
	{
		va_start(arg_list, format);
		Len = valen(format, arg_list);
		va_end(arg_list);
		Len = Len + 10;
		
		instr = drealloc(instr, Len);
		
		va_start(arg_list, format);
		vsprintf(instr, format, arg_list);
		va_end(arg_list);
	}
	return(instr);
}

/******************************************************************************
*       unsigned valen(char *format, va_list arg_list)
*		Returns the combined total length of all args in a va_list.
*
*	char *format		The formating string.
*	va_list arg_list	The va_list to measure.
*
*       Updated :
******************************************************************************/	
unsigned valen(char *format, va_list arg_list)
{
	unsigned Len = 1;
	char TmpIntBuffer[255];
	char ArgTypes[] = "scdlxufe ";
	unsigned i;
	
	if(format)
	{	
		for(i=0;i<dstrlen(format);i++)
		{
			if(format[i] == '%')
			{
				do
				{
					i++;
					if(strchr(ArgTypes, format[i]))
					{
						switch(format[i])
						{
							case 's':
								Len = Len + dstrlen(va_arg(arg_list, char *));
								break;
							case 'c':
								Len = Len + 2;
								break;
							case 'd':
								sprintf(TmpIntBuffer, "%d", va_arg(arg_list, int));
								Len = Len + dstrlen(TmpIntBuffer);
								break;
							case 'l':
								sprintf(TmpIntBuffer, "%ld", va_arg(arg_list, long));
								Len = Len + dstrlen(TmpIntBuffer);
								break;
							case 'x':
								sprintf(TmpIntBuffer, "%x", va_arg(arg_list, int));
								Len = Len + dstrlen(TmpIntBuffer);
								break;
							case 'u':
								sprintf(TmpIntBuffer, "%u", va_arg(arg_list, unsigned));
								Len = Len + dstrlen(TmpIntBuffer);
								break;
							case 'e':
								sprintf(TmpIntBuffer, "%d", va_arg(arg_list, int));
								Len = Len + dstrlen(TmpIntBuffer);
								break;
							case 'f':
								sprintf(TmpIntBuffer, "%f", va_arg(arg_list, double));
								//printf("---[%s]-[%d]\n", TmpIntBuffer, dstrlen(TmpIntBuffer));
								Len = Len + dstrlen(TmpIntBuffer);
								break;
							case ' ':
								Len++;
								break;
						}
					}
					Len++;
				}while(format[i] != '\0' && !strchr(ArgTypes, format[i]));
			} else {
				Len++;
			}
		}
		Len++;
		//printf("---[Len:%d]\n", Len);
	}
	return(Len);
}

/******************************************************************************
*       char * dstrsep(char **string, char *delim)
*		Provides the strsep like function to systems that do not support 
*		strsep. Function supports multiple byte deleminators. Key 
*		difference is that it doesnt affect StringToParse so is 
*		free() friendly.
*
*
*	Args :
*		char * String		String pointer to search on.
*		char * Delim		String to deleminate on.
*		char * NextPointrt	Pointer to set with 'Next' pointer.
*
*       Updated :
******************************************************************************/
char * dstrsep(char * StringToParse, char * Delim, char ** NextPointer)
{
	char * Start = 0;
	char * Pos = 0;
	
	Start = StringToParse;
	
	if(Start && Delim)
	{
		if(dstrstr(Start, Delim))
		{
			Pos = (char *) dstrstr(Start, Delim);
			if(Pos)
			{
				*Pos = 0;
				if(*NextPointer != (char *) -1)
				{
					*NextPointer = dstradvance(Pos, dstrlen(Delim));
				}
			}
			return(Start);
		} else {
			return(0);
		}
	}
	return(0);
}

