/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : mysqldb.c
* 		Functions for performing MySQL db connections and result handling.
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*		12/15/2003 - Added MyDebug() statements for debugging of SQL
*			     functions.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql/mysql.h>

#include "../Commoninclude.h"

extern int LOG_SQL;

/******************************************************************************
*       struct DBR * dbQuery(char * query, struct SQLCFG * sqlcfg)
*		Connects to specified MySQL server, performs query, stores 
*		result and disconnects from server. Returns struct containing
*		number of results and results.
*
*	Args :
*		char * query		SQL Query string.
*		struct SQLCFG * sqlcfg	SQL Config struct.
*
*       Updated :
******************************************************************************/
struct DBR * dbQuery(char * query, struct SQLCFG * sqlcfg)
{
	MYSQL mysql;
	struct DBR * dbr = 0;
	
	mysql_init(&mysql);
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery]-[IP:%s]-[User:%s]-[DB:%s]-[Port:%ld]", 
					     sqlcfg->IP, sqlcfg->user, sqlcfg->db, sqlcfg->port);
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery]-[Query:%s]", query);
	
	if(query)
	{
		if(mysql_real_connect(&mysql, sqlcfg->IP, sqlcfg->user, sqlcfg->passwd, sqlcfg->db, sqlcfg->port, sqlcfg->unixsocket, 0))
		{
        		if(!mysql_query(&mysql, query))
        		{
        			dbr = (struct DBR *) malloc(sizeof(struct DBR));
        			memset(dbr, 0, sizeof(struct DBR));
        			
    				if(dbr)
    				{
        				dbr->results = mysql_store_result(&mysql);
        		        
        		        	if(dbr->results)
        		        	{
        		        		dbr->num = mysql_num_rows(dbr->results);
        		        	} else {
        		        		dbr->num = 0;
        		        	}
        		        	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery]-[Records:%u]", dbr->num);
        		        }
			} else {
				MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery]-[Invalid Query]");
			}
			mysql_close(&mysql);
		} else {
			MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery]-[Unable to connect]");
		}
	}
	
	return(dbr);
}

/******************************************************************************
*       struct DBR * dbQuery_P(struct DBR * dbr, char * query)
*		Same functionality as dbQuery() but it assumes the DB 
*		connection exists and it does not close the connection on exit.
*
*	Args :
*		MYSQL mysql		The MySQL connection handle.
*		char * query            SQL Query string.
*
*       Updated :
******************************************************************************/
struct DBR * dbQuery_P(MYSQL * mysql, char * query)
{
	struct DBR * dbr = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery_P]-[enter]");
	
	if(query)
	{
		MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery_P]-[Query:%s]", query);
		
		if(!mysql_query(mysql, query))
		{
			dbr = (struct DBR *) drealloc((char *) dbr, sizeof(struct DBR));
			
			dbr->results = mysql_store_result(mysql);
			
			if(dbr->results)
			{
				dbr->num = mysql_num_rows(dbr->results);
			} else {
				dbr->num = 0;
			}
			
			MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery_P]-[Records:%u]", dbr->num);
		} else {
			MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery_P]-[Invalid query]");
		}
	} else {
		MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery_P]-[No query specified]");
	}
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbQuery_P]-[exit]");
	
	return(dbr);
}

/******************************************************************************
*       MYSQL * dbConnect(struct SQLCFG * sqlcfg)
*		Connects to specified server. Returns DB Result which contains
*		MySQL connection handle.
*
*	Args :
*		struct SQLCFG * sqlcfg		SQL Config struct.
*
*       Updated :
******************************************************************************/
MYSQL * dbConnect(MYSQL * mysql, struct SQLCFG * sqlcfg)
{
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbConnect]-[enter]");
	
	if(sqlcfg)
	{
		MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbConnect]-[IP:%s]-[User:%s]-[DB:%s]-[Port:%ld]", 
						     sqlcfg->IP, sqlcfg->user, sqlcfg->db, sqlcfg->port);
	
		mysql_init(mysql);
		if(!mysql_real_connect(mysql, sqlcfg->IP, sqlcfg->user, sqlcfg->passwd, sqlcfg->db, sqlcfg->port, NULL, 0))
		{
			MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbConnect]-[Unable to connect to SQL server]");
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbConnect]-[exit]");
	
	return(mysql);
}	

/******************************************************************************
*       void dbClose(MYSQL mysql)
*		Closes MySQL connection.
*
*	Args :
*		struct DBR * dbr	SQL Config struct.
*
*       Updated :
******************************************************************************/
void dbClose(MYSQL * mysql)
{
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbClose]-[enter]");
	
	if(&mysql)
	{
		mysql_close(mysql);
	}
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbClose]-[exit]");
	
}

/******************************************************************************
*       MYSQL_ROW dbResult(struct DBR * dbr)
*		Returns one row from DB Result.
*
*	Args :
*		struct DBR * dbr	DB Result struct.
*
*       Updated :
******************************************************************************/
MYSQL_ROW dbResult(struct DBR * dbr)
{
	MYSQL_ROW myrow;
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbResult]-[enter]");
	
	if(dbr)
	{
		if(dbr->num)
		{
			myrow = mysql_fetch_row(dbr->results);
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbResult]-[exit]");
	
	return(myrow);
}

/******************************************************************************
*       struct DBR * dbFree(struct DBR * dbr)
*		Frees both the DB Results and the Db result struct.
*
*	Args :
*		struct DBR * dbr	DB Result struct.
*
*       Updated :
******************************************************************************/	
struct DBR * dbFree(struct DBR * dbr)
{	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbFree]-[enter]");
	
	if(dbr)
	{
		if(dbr->results)
		{
			MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbFree]-[freeing:%u]", dbr->num);
			mysql_free_result(dbr->results);
		}
		dbr = (struct DBR *) dfree((char *)dbr);
	}
	
	MyDebug(__LINE__, __FILE__, LOG_SQL, "[dbFree]-[exit]");
	
	return(dbr);
}
