/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : Socket.c
*		Socket functions used through out MIDAS. These functions are
*		only meant for low-ascii traffic and rely on a EOL character
*		to designate when a transmission is finished.  
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*		11/20/2003 - Made Windows portable.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <sys/time.h>

#ifdef __WIN32__
#include <winsock.h>
#include <winsock2.h>

// Windows doesnt like read/write so we use send and recv instead.
#define write(x, y, z) send(x, y, z, 0)
#define read(x, y, z) recv(x, (void *) y, z, 0) 
#else
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/socket.h>
#endif

#include "../Commoninclude.h"

char MIDAS_EOL[] = "\xFF";

int MAXSOCKETTIMEOUT = 120;
int SOCKTIMEOUT = 1;

extern int LOG_SOCKET;

/******************************************************************************
*       char * RecvFromSocket(int sfd, char * EOL, int Crypt)
*		Receives data from socket.
*
*	Args :
*		int sfd		Socket handle.
*		char * EOL	The End Of Line/Transimission Character.
*		int Crypt	Toggle Decryption.
*
*       Updated :
******************************************************************************/
char * RecvFromSocket(int sfd, char * EOL, int Crypt)
{	
	char ReadBuffer[65534];
	char * OutStr = 0;
	char * tOutStr = 0;
  	fd_set sel_sock;
        struct timeval tv;
	int len;
	int MaxSocketTimeOut = 0;
	int sock_status;
	unsigned LoopCount = 0;
	
	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Enter]");
		
	MaxSocketTimeOut = MAXSOCKETTIMEOUT;
	
	while(1 == 1)

	{
		FD_ZERO(&sel_sock);
		FD_SET(sfd, &sel_sock);
	
		tv.tv_sec = SOCKTIMEOUT;
		tv.tv_usec = 0; 
	        
	        sock_status = select(sfd + 1, &sel_sock, NULL, NULL, &tv);
		//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Sock_Status:%d]", sock_status);
		
		LoopCount++;
		if(LoopCount > 32768)
		{
			MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Excessive Looping]");
			LoopCount = 0;
		}
		
	        if(sock_status > 0)
	        {
	        	len = read(sfd, &ReadBuffer, sizeof(ReadBuffer) - 1);
	        	
			//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Recv Len:%d]", len);
	                if(len > 0)
	                {
				ReadBuffer[len] = 0;
	   			OutStr = dstrcat(OutStr, ReadBuffer);
	                	
	                	MaxSocketTimeOut = MAXSOCKETTIMEOUT;
	                	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Recvd:%d]", dstrlen(OutStr));
	                } else {
	                	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Failed to read anything]-[%d]", MaxSocketTimeOut);
	                	MaxSocketTimeOut = MaxSocketTimeOut - SOCKTIMEOUT;
	                }
	        } else {
	        	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Nothing to read yet-[%d]", MaxSocketTimeOut);
	        	MaxSocketTimeOut = MaxSocketTimeOut - SOCKTIMEOUT;
	        }
	        
	        FD_ZERO(&sel_sock);
	        
	        if(dstrstr(OutStr, EOL))
	        {
	  		//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Got EOL]");
	        	break;
	        }
	        
	        if(MaxSocketTimeOut < 1)
	        {
	        	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Timeout]");
	        	break;
	        }
	};
	OutStr = dsubstrreplace(OutStr, EOL, "");
	
	if(Crypt)
	{
		tOutStr = Decrypt(OutStr);
		OutStr = dfree(OutStr);
		OutStr = tOutStr;
	}
	
	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[RecvFromSocket]-[Exit]");
	return(OutStr);
}

/******************************************************************************
*       int SendToSocket(int sfd, char * msg, char * EOL, int Crypt)
*		Sends a string buffer to specified socket.
*
*	Args :
*		int sfd		Socket handle.
*		char * msg	Buffer to send.
*		char * EOL	End of Line/Transmission character.
*		int Crypt	Toggle encryption.
*
*       Updated :
******************************************************************************/
int SendToSocket(int sfd, char * msg, char * EOL, int Crypt)
{
  	int status = 0;
  	int sock_status = 0;
  	unsigned SentLen = 0;
  	fd_set sel_sock;
        struct timeval tv;
	int MaxSocketTimeOut;
	char * SendBuf = 0;
	char * LocalMsg = 0;
	char * tLocalMsg = 0;
	unsigned MsgLen = 0;
	
	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToSocket]-[Enter]");
	
	MaxSocketTimeOut = MAXSOCKETTIMEOUT;
	
	LocalMsg = dstrcpy(LocalMsg, msg);
	
	if(Crypt)
	{
		tLocalMsg = Encrypt(LocalMsg);
		LocalMsg = dfree(LocalMsg);
		LocalMsg = tLocalMsg;		
	}
	
	if(EOL)
	{
		LocalMsg = dstrcat(LocalMsg, EOL);
		MsgLen = dstrlen(LocalMsg);
		SendBuf = LocalMsg;
	}
	
	while(sock_status != -1 && status != -1)
	{
		FD_ZERO(&sel_sock);
		FD_SET(sfd, &sel_sock);

		tv.tv_sec = SOCKTIMEOUT;
		tv.tv_usec = 0;
		
		sock_status = select(sfd + 1, NULL, &sel_sock, NULL, &tv);
		//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToSocket]-[Sock Status: %d]", sock_status);
		
		if(sock_status > 0)
		{
			status = write(sfd, SendBuf, dstrlen(SendBuf));
			
			if(status > 0)
			{
				SendBuf = dstradvance(SendBuf, status);
				SentLen = SentLen + status;
			}
			
			//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToSocket]-[Send Status:%d]-[%u]-[%u]", status, SentLen, MsgLen);
			MaxSocketTimeOut = MAXSOCKETTIMEOUT;
		} else {
			//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToSocket]-[Socket not ready to send]-[%d]", MaxSocketTimeOut);
			MaxSocketTimeOut = MaxSocketTimeOut - SOCKTIMEOUT;
		}
		
		FD_ZERO(&sel_sock);
		
		if(!SendBuf)
		{
			//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToSocket]-[End of SendBuf]");
			break;
		}
		
		if(SentLen >= MsgLen)
		{
			break;
		}
		
		if(MaxSocketTimeOut < 1)
	        {
	        	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToSocket]-[Timeout]");
	        	break;
	        }		
	}
	
	LocalMsg = dfree(LocalMsg);
		
	//MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToSocket]-[Exit]");
	return(status);
}
