/*****************************************************************************
* Copyright (c) 2004 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : ping.c
*		Functions to perform ICMP ping checks
*
*       Updated :
*		02/19/04 - Fixed icmp id issue. Now uses the pid of process as 
*			   ID. 
*		02/19/04 - Fixed Return packet check now makes sure its a 
*			   ICMP_ECHOREPLY.
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <time.h>
#include <string.h>

#ifdef __WIN32__
#include <winsock2.h>
#include <ws2tcpip.h>
#include "ip_icmp.h"
#else
#include <netdb.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#endif

#include "../Commoninclude.h"

// [TCC] Win32 we CANT build the IP header so we have to use a different struct 
//       when building the packet to send. Problem caused : icmp_type gets over
//       written with the value 0x45 (Type 69).
#ifdef __WIN32__
struct ICMPPacket {
	struct icmp icmp;
};
#else
struct ICMPPacket {
	struct ip ip;
	struct icmp icmp;
};
#endif

// [TCC] On the return packet we will always get the IP and ICMP headers.
struct RecvICMPPacket {
	struct ip ip;
	struct icmp icmp;
};

extern int LOG_INFO;

// [TCC] We remove the MyDebug()'s calls by default. Comment out this line 
//       to turn then back on.
#define MyDebug(...);

struct ICMPResponse * ping (char * DSTIP, long Pings, unsigned TTL)
{
	int s;
	int Status = 0;
	struct ICMPPacket * ICMPPacket = 0;
	struct ICMPResponse * ICMPResponse = 0;
	long int TripTime = 0;
	long i;

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ping]-[Enter]-[%s]-[%ld]-[%u]", DSTIP, Pings, TTL);
	
	if(Pings)
	{
		for(i=0;i<Pings;i++)
		{
			s = BuildSocket();
			if(s)
			{
				ICMPPacket = BuildICMPPacket(ICMPPacket, DSTIP);
				if(ICMPPacket)
				{
					if(!ICMPResponse)
					{
						ICMPResponse = (struct ICMPResponse *) drealloc((char *) ICMPResponse, sizeof(struct ICMPResponse));
					}
					
					ICMPResponse->PacketsSent++;
					Status = send_ping(s, ICMPPacket, DSTIP);
					if(Status != -1)
					{
						TripTime = recv_ping(s, ICMPPacket, TTL);

						if(TripTime != -1)
						{
							ICMPResponse->PacketsRecv++;
							
							if(TripTime > ICMPResponse->Trip.Max)
							{
								ICMPResponse->Trip.Max = TripTime;
							}
							
							if(TripTime < ICMPResponse->Trip.Min || i == 0)
							{
								ICMPResponse->Trip.Min = TripTime;
							}
							
							ICMPResponse->Trip.Avg = ((ICMPResponse->Trip.Avg + TripTime) / 2);
						} else {
							ICMPResponse->PacketsLost++;
						}
					} else {
						ICMPResponse->PacketsLost++;
					}
				}
				ICMPPacket = (struct ICMPPacket *) dfree((char *)ICMPPacket);
			}
#ifdef __WIN32__
			closesocket(s);
#else
			close(s);
#endif
		}
	}

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ping]-[exit]");
	
	return(ICMPResponse);
}

int BuildSocket()
{
	int s = 0;
	int on = 1;

// [TCC] Have to use WSASocket. This is to specify the procotol (ICMP) since we 
//       can not generate the IP header ourselves.	
#ifdef __WIN32__
	on = 0;
	s = WSASocket(AF_INET, SOCK_RAW, IPPROTO_ICMP, 0, 0, 0);
#else
	s = socket(AF_INET, SOCK_RAW, 1);
	if(s)
	{
		setsockopt(s, IPPROTO_IP, IP_HDRINCL, (void *) &on, sizeof(on));
	}
#endif
	return(s);
}


struct ICMPPacket * BuildICMPPacket(struct ICMPPacket * ICMPPacket, char * DSTIP)
{
	struct hostent * host;
	char * tIP = 0;
	
	if(DSTIP)
	{
		ICMPPacket = (struct ICMPPacket *) drealloc((char *) ICMPPacket, sizeof(struct ICMPPacket));
	
		if(ICMPPacket)
		{

			ICMPPacket->icmp.icmp_type = ICMP_ECHO;
			ICMPPacket->icmp.icmp_code = 0;
			ICMPPacket->icmp.icmp_cksum = 0;
			ICMPPacket->icmp.icmp_id = getpid() & 0xFFFF;
			ICMPPacket->icmp.icmp_seq = 0;
			ICMPPacket->icmp.icmp_cksum = in_cksum((u_short *)&ICMPPacket->icmp, sizeof(struct icmp));
#ifdef __WIN32__
			host = 0;
			tIP = 0;
#else	
			// [TCC] On all systems BUT Win32 we need to build the IP header.		
			ICMPPacket->ip.ip_v = 4;
			ICMPPacket->ip.ip_hl = 5;
			ICMPPacket->ip.ip_tos = 0;
#ifdef __FreeBSD__
                        //FreeBSD requires ip_len in host bit order
			ICMPPacket->ip.ip_len = sizeof(struct ICMPPacket);
#else
			ICMPPacket->ip.ip_len = htons(sizeof(struct ICMPPacket));
#endif
			ICMPPacket->ip.ip_id = rand() & 0xFFFF;
			ICMPPacket->ip.ip_off = htons(0);
			ICMPPacket->ip.ip_ttl = 0xFF;
			ICMPPacket->ip.ip_p = IPPROTO_ICMP;
			ICMPPacket->ip.ip_sum = 0;
			ICMPPacket->ip.ip_src.s_addr = 0;
			
			host = gethostbyname(DSTIP);
			if(host)
			{
				tIP = dstrcpy(tIP,  (char *) inet_ntoa(*(struct in_addr *) host->h_addr_list[0]));
				ICMPPacket->ip.ip_dst.s_addr = inet_addr(tIP);
				tIP = dfree(tIP);
				ICMPPacket->ip.ip_sum = in_cksum((u_short *)&ICMPPacket->ip, ntohs(ICMPPacket->ip.ip_len));
			} else {
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[BuildICMPPacket]-[Invalid Host:%s]", DSTIP);
				ICMPPacket = (struct ICMPPacket *) dfree((char *)ICMPPacket);
			}
#endif
		}
	}
	
	return(ICMPPacket);
}

long int recv_ping(int s, struct ICMPPacket * ICMPPacket, unsigned TTL)
{
	struct RecvICMPPacket * RecvICMPPacket = 0;
	int RecvCount = 0;
	struct timeval tv;
	struct timeval Starttv;
	struct timeval Returntv;
	long int TripTime = -1;
	int sock_status = 0;
	fd_set sel_sock;
	unsigned StartTime = 0;
	
	gettimeofday(&Starttv, NULL);	

	StartTime = time(NULL);
        
	for(;;)
	{
		FD_ZERO(&sel_sock);
        	FD_SET(s, &sel_sock);

		tv.tv_sec = 1;
        	tv.tv_usec = 0;
		
		sock_status = select(s + 1, &sel_sock, NULL, NULL, &tv);
		
		if(sock_status > 0)
		{
			RecvICMPPacket = (struct RecvICMPPacket *) realloc((char *) RecvICMPPacket, sizeof(struct RecvICMPPacket));
			                                                   
			RecvCount = recvfrom(s, (void *) RecvICMPPacket, sizeof(struct RecvICMPPacket), 0, NULL, NULL);
			
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[recv_ping]-[RecvCount:%d]", RecvCount);
			
 			if(RecvCount > 0) 
 			{
 				if(RecvICMPPacket->icmp.icmp_type == ICMP_ECHOREPLY &&
 				   RecvICMPPacket->icmp.icmp_id == ICMPPacket->icmp.icmp_id)
 				{
					gettimeofday(&Returntv, NULL);
					
					if(Returntv.tv_usec >= Starttv.tv_usec)
					{
						TripTime = Returntv.tv_usec - Starttv.tv_usec;
					} else {
						TripTime = (Returntv.tv_usec + 1000000) - Starttv.tv_usec;	
					}
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[recv_ping]-[%ld]-[%ld]-[%u]",  Starttv.tv_usec, Returntv.tv_usec, TripTime);
					
					RecvICMPPacket = (struct RecvICMPPacket *) dfree((char*) RecvICMPPacket);
					break;
				} else {
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[recv_ping]-[Invalid Packet: id:%ld]\n", RecvICMPPacket->icmp.icmp_id);
				}
			}
			
			RecvICMPPacket = (struct RecvICMPPacket *) dfree((char*) RecvICMPPacket);
		}
		
		if((time(NULL) - StartTime) > TTL)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[recv_ping]-[Timeout]");
			
			break;
		}
	}
    	return(TripTime);
}

int send_ping(int s, struct ICMPPacket * ICMPPacket, char * DSTIP)
{
	int Status = 0;
	struct sockaddr_in dest;
	struct hostent * host;
	
	if(ICMPPacket)
	{
		
		host = gethostbyname(DSTIP);
		if(host)
		{
			memset((char *)&dest, 0, sizeof(struct sockaddr_in));
			dest.sin_family = AF_INET;
			dest.sin_addr = *(struct in_addr *) *host->h_addr_list;
			memset(&(dest.sin_zero),0, 8);
			
			Status = sendto(s, (void *) ICMPPacket, sizeof(struct ICMPPacket), 0, (struct sockaddr*)&dest, sizeof (dest));
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[send_ping]-[Status:%d]", Status);
		} else {
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[send_ping]-[Invalid hostname/ip]");
		}
		
	}	
	
	return(Status);
}

unsigned short in_cksum(u_short *buf, int N)
{
	unsigned sum = 0;
	unsigned int odd;

	while(N > 1) 
	{
		sum += *buf++;
		N -= 2;
	}
	
	if(N == 1) 
	{
        	odd = 0;
        	*((unsigned int *) &odd) = *(unsigned char *) buf;
		sum += odd;
	}
	
	sum = (sum >> 16) + (sum & 0xFFFF);
	sum += (sum >> 16);

	return (unsigned short) ~sum;
}
