/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASc
*       Filename : check_network.c
*		Functions for handling network checks (TCP, UDP, ICMP, Ping, 
*		SNMP).
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*		11/25/2003 - Added support for TCP connection to send a string
*			     on connection. It will record anything sent back.
*		12/08/2003 - Check_network_ICMP now uses internal ping() 
*			     function instead of piping out to the ping 
*			     command.
*		12/09/2003 - Fixed issues with WinSocket support for the TCP/UDP
*			     check functions.
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>

#ifdef __WIN32__
#include <winsock.h>
#include <winsock2.h>
#define COUNTARG "-n "
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#define COUNTARG "-c"
#endif

#include "includes.h"
#include "extern.h"
#include "../Common/Commoninclude.h"

struct 	SNMPDB {
	int 	HID;
	int	CID;
	int	OptionNum;
	float	Value;
	};

/******************************************************************************
*       char * Check_Network_TCP(struct XMLLeaf * CheckXMLTree)
*		Performs a TCP connect() check. 
*
*	Args :
*		struct XMLLeaf * CheckXMLTree	XML linked list containing
*						check options.
*
*       Updated :
*		11/25/2003 - Now returns a RED when unable to resolve 
*			     IP/Hostname.
******************************************************************************/
char * Check_Network_TCP(struct XMLLeaf * CheckXMLTree)
{
	int sock;
	int connected;
	char * Response = 0;
	char * tResponse = 0;
	char * RecvBuff = 0;
	char * IP = 0;
	int Port = 0;
        struct sockaddr_in addr;
        struct hostent * hostinfo = 0;
	fd_set fds;
	struct timeval tv;
	int sock_status = 0;
	int error = 0;
	int error_len = 0;
	char * SendBuff = 0;
	
	IP = ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.IP");
	Port = Myatoi(ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.PORT"));
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Network_TCP]-[IP:%s]-[Port:%d]", IP, Port);
	
	if(IP && Port)
	{
		Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
				    ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.HID"),
				    ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.CID"));
				    
        	sock = socket(AF_INET, SOCK_STREAM, 0);
        	if(sock != -1) 
        	{
			MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_TCP]-[Socket Built]");
			hostinfo = gethostbyname(IP);
			
			if(hostinfo)
			{
				addr.sin_family = AF_INET;
				addr.sin_port = htons(Port);
				addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
				memset(&(addr.sin_zero), '\0',8);
			
				fcntl(sock, F_SETFL, O_NONBLOCK);
	                	connected = connect(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr));
	                
	                	FD_ZERO(&fds);
	                	FD_SET(sock, &fds);
	                	
	                	tv.tv_sec = 5;
	                	tv.tv_usec = 0;
	                	
	                	sock_status = select(sock+1, NULL, &fds, NULL, &tv);
	                	
				if(sock_status > 0)
				{
					error_len = sizeof(error);
					getsockopt(sock, SOL_SOCKET, SO_ERROR, (char *)&error, &error_len);
					
					if(error == 0)
					{
						MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_TCP]-[Connected]");
						
						if(!Myatoi(ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.CONNECTONLY")))
						{
							if(ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.OPTION.SEND"))
							{
								SendBuff = ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.OPTION.SEND");
								SendBuff = UnEscapeChars(SendBuff);
							} else {
								SendBuff = dstrcpy(SendBuff, "\r\n");
							}
							
							SendToSocket(sock, SendBuff, "", 0);
							SendBuff = dfree(SendBuff);
							
							RecvBuff = RecvFromSocket(sock, "\n", 0);
							if(RecvBuff)
							{
								RecvBuff = CleanXMLValueString(RecvBuff);
								
								tResponse = dsprintf(tResponse, "<DETAIL>Response : %s</DETAIL><STATUS>GREEN</STATUS>", RecvBuff);
								Response = dstrcat(Response, tResponse);
								tResponse = dfree(tResponse);
							} else {
								Response = dstrcat(Response, "<DETAIL>Connected But no Response</DETAIL><STATUS>RED</STATUS>");
							}
							RecvBuff = dfree(RecvBuff);
						} else {
							Response = dstrcat(Response, "<DETAIL>Connected</DETAIL><STATUS>GREEN</STATUS>");
						}
					} else {
						MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_TCP]-[Failed to Connect]");
						Response = dstrcat(Response, "<DETAIL>Failed to Connect</DETAIL><STATUS>RED</STATUS>");
					}
    				} else {
    					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_TCP]-[Failed to Connect]");
    					Response = dstrcat(Response, "<DETAIL>Failed to Connect</DETAIL><STATUS>RED</STATUS>");
    				}

    			} else {
    				MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_TCP]-[Invalid IP/Hostname]");
    				Response = dstrcat(Response, "<DETAIL>Invalid IP or Hostname</DETAIL><STATUS>RED</STATUS>");	
    			}
    		} else {
			MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_TCP]-[Failed to Connect]");
    			Response = dstrcat(Response, "<DETAIL>Failed to Connect</DETAIL><STATUS>RED</STATUS>");	
    		}
		
#ifdef __WIN32__
		closesocket(sock);
#else
		close(sock);
#endif

		Response = dstrcat(Response, "</CHECK>");
	}

	return(Response);
}

/******************************************************************************
*       char * Check_Network_UDP(struct XMLLeaf * CheckXMLTree)
*		Performs a UDP connect() check. 
*
*	Args :
*		struct XMLLeaf * CheckXMLTree	XML linked list containing
*						check options.
*
*       Updated :
*		11/25/2003 - Now returns a RED when unable to resolve 
*			     IP/Hostname.
******************************************************************************/
char * Check_Network_UDP(struct XMLLeaf * CheckXMLTree)
{
	int sock;
	int connected;
	char * Response = 0;
	char * IP = 0;
	int Port = 0;
        struct sockaddr_in addr;
	struct hostent * hostinfo = 0;

	IP = ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.IP");
	Port = Myatoi(ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.PORT"));
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Network_UDP]-[IP:%s]-[Port:%d]", IP, Port);
	
	if(IP && Port)
	{
		Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
				    ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.HID"),
				    ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.CID"));
				    
        	sock = socket(AF_INET, SOCK_DGRAM, 0);
        	if(sock != -1) 
        	{
			MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_UDP]-[Socket Built]");

			hostinfo = gethostbyname(IP);
			
			if(hostinfo)
			{
				addr.sin_family = AF_INET;
				addr.sin_port = htons(Port);
				addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
				memset(&(addr.sin_zero), '\0',8);

	                	connected = connect(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr));
				if(connected != -1) 
				{
					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_UDP]-[Connected]");
					Response = dstrcat(Response, "<DETAIL>Connected</DETAIL><STATUS>GREEN</STATUS>");
    				} else {
    					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_UDP]-[Failed to Connect]");
    					Response = dstrcat(Response, "<DETAIL>Connected</DETAIL><STATUS>RED</STATUS>");
    				}
    			} else {
    				MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_TCP]-[Invalid IP/Hostname]");
    				Response = dstrcat(Response, "<DETAIL>Invalid IP or Hostname</DETAIL><STATUS>RED</STATUS>");	
    			}

    		} else {
			MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_UDP]-[Failed to Connect]");
    			Response = dstrcat(Response, "<DETAIL>Connected</DETAIL><STATUS>RED</STATUS>");	
    		}
#ifdef __WIN32__
		closesocket(sock);
#else
		close(sock);
#endif
		Response = dstrcat(Response, "</CHECK>");
	}

	return(Response);
}

/******************************************************************************
*       char * Check_Network_ICMP(struct XMLLeaf * CheckXMLTree)
*		Performs a ICMP (ping) check. Will use the CONFIG.CML.PING
*		from MIDASc.cf.
*
*	Args :
*		struct XMLLeaf * CheckXMLTree	XML linked list containing
*						check options.
*
*       Updated :
******************************************************************************/
char * Check_Network_ICMP(struct XMLLeaf * CheckXMLTree)
{
	char * IP = 0;
	char * Detail = 0;
	char * Response = 0;
	char * Stats = 0;
	char * Status = 0;
	struct ICMPResponse * ICMPResponse = 0;
	int Count = 0;
	int TTL = 0;
		
	IP = ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.IP");
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "Check_Network_ICMP]-[Enter]-[%s]", IP);

	if(IP)
	{		
		Count = Myatoll(ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.OPTION.PING.COUNT"));
		if(!Count)
		{
			Count = 5;
		}
		
		TTL = Myatoll(ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.OPTION.PING.TTL"));
		if(!TTL)
		{
			TTL = 3;
		}
		
		ICMPResponse = ping(IP, Count, TTL);
		if(ICMPResponse)
		{
			MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_ICMP]-[%ld,%ld,%ld]-[%.2f,%.2f,%.2f]",
							       ICMPResponse->PacketsSent, ICMPResponse->PacketsRecv, ICMPResponse->PacketsLost,
							       (float) ((float) ICMPResponse->Trip.Max / (float) 1000),
						  	       (float) ((float) ICMPResponse->Trip.Avg / (float) 1000),
						  	       (float) ((float) ICMPResponse->Trip.Min / (float) 1000));
						  	       
			Detail = dsprintf(Detail, "Ping Results:\n Packets (Sent,Recv,Lost) : %ld, %ld, %ld\n Trip Time (Max,Avg,Min) : %.2fms, %.2fms, %.2fms\n Packet Loss : %d%%", 
						  ICMPResponse->PacketsSent, ICMPResponse->PacketsRecv, ICMPResponse->PacketsLost,
						  (float) ((float) ICMPResponse->Trip.Max / (float) 1000),
						  (float) ((float) ICMPResponse->Trip.Avg / (float) 1000),
						  (float) ((float) ICMPResponse->Trip.Min / (float) 1000),
						  (100 / ICMPResponse->PacketsSent) * ICMPResponse->PacketsLost);
			
			Stats = dsprintf(Stats, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>0</ORDER><LABEL>Round Trip</LABEL><DATAVAL>%.2f</DATAVAL></STAT><STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>1</ORDER><LABEL>Packet Loss</LABEL><DATAVAL>%d</DATAVAL></STAT>", 
					        ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.CID"),
					        ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
					        (float) ((float) ICMPResponse->Trip.Avg / (float) 1000),
					        ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.CID"),
					        ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
					        ((100 / ICMPResponse->PacketsSent) * ICMPResponse->PacketsLost));

			if(ICMPResponse->PacketsRecv == 0)
			{
				Status = dstrcpy(Status, "RED");
			} else if(ICMPResponse->PacketsRecv != 0 && ICMPResponse->PacketsRecv < ICMPResponse->PacketsSent) {
				Status = dstrcpy(Status, "YELLOW");
			} else if(ICMPResponse->PacketsRecv == ICMPResponse->PacketsSent) {
				Status = dstrcpy(Status, "GREEN");
			} else {
				Status = dstrcpy(Status, "RED");
			}
			
		} else {
			Detail = dsprintf(Detail, "Error Sending pings to %s\n", IP);
			Status = dstrcpy(Status, "RED");
		}
		
		ICMPResponse = (struct ICMPResponse *) dfree((char *)ICMPResponse);
		
	} else {
		Detail = dstrcpy(Detail, "No IP specified for ping check\n");
		Status = dstrcpy(Status, "RED");
	}	
	
	if(!Stats)
	{
		Stats = dstrcpy(Stats, "");
	}
			
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID><DETAIL>%s</DETAIL><STATUS>%s</STATUS>%s</CHECK>", 
				      ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.HID"),
				      ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.CID"),
				      Detail, Status, Stats);
		
	Stats = dfree(Stats);
	Detail = dfree(Detail);
	Status = dfree(Status);
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Network_ICMP]-[Exit]");
	
	return(Response);
}

/******************************************************************************
*       char * Check_Network_SNMP(struct XMLLeaf * CheckXMLTree)
*		Performs a SNMP check on a specific nemeric OID.
*
*	Args :
*		struct XMLLeaf * CheckXMLTree	XML linked list containing
*						check options.
*
*       Updated :
*		11/07/2003 - Removed GRAPHTYPE tag from STAT
******************************************************************************/
char * Check_Network_SNMP(struct XMLLeaf * CheckXMLTree)
{
	struct XMLLeaf * LocalLeaf;
	struct XMLLeaf * CheckLeaf;
	struct SNMPData * SnmpData = 0;
	char * Response = 0;
	char * tResponse = 0;
	char * Detail = 0;
	char * tDetail = 0;
	int Status = 1;
	char * IP = 0;
	char * COMMUNITY = 0;
	char * OID = 0;
	char * LOGIC = 0;
	char * EXPECT = 0;
	char * FORMAT = 0;
	char * FormatData = 0;
	char * tSTAT = 0;
	char * STAT = 0;
	char * LABEL = 0;
	int OptionCount = 1;
	char * SNMPTYPE = 0;
	char * HID = 0;
	char * CID = 0;
	float PrevSNMPValue = 0;
	float CurSNMPValue = 0;
	float  Counter32Max = 3.40282347e+38F;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Network_SNMP]-[Enter]");
	
	HID =  ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.HID");
	CID = ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.CID");
		  
  	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", HID, CID);

  	LocalLeaf = CheckXMLTree;
  	CheckLeaf = CheckXMLTree;
  	
  	IP = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.IP");
  	
  	if(LocalLeaf)
  	{
  		while(LocalLeaf)
  		{
			CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP"), 1);
			if(CheckLeaf)
			{
				
				COMMUNITY = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.COMMUNITY");
				OID = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.OID");
				LOGIC = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.LOGIC");
				EXPECT = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.EXPECT");
				FORMAT = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.FORMAT");

				if(IP && COMMUNITY && OID)
				{
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Network_SNMP]-[IP:%s]-[COM:%s]-[OID:%s]", IP, COMMUNITY, OID);
					
					SnmpData = GetSNMPResponse(IP, COMMUNITY, OID);
					if(SnmpData)
					{
						if(SnmpData->Status == 1)
						{
							SNMPTYPE = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.TYPE");
							if(SNMPTYPE)
							{
								MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Network_SNMP]-[SNMPTYPE:%s]", SNMPTYPE);
								
								if(!dstrcasecmp(SNMPTYPE, "COUNTER"))
								{
									PrevSNMPValue = Check_Network_SNMP_ReturnPrevResult(HID, CID, OptionCount);
									Check_Network_SNMP_UpdateDB(HID, CID, OptionCount, SnmpData->Data);
									
									MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Network_SNMP]-[Prev:%f]-[Cur:%s]", PrevSNMPValue, SnmpData->Data);
									
									if(PrevSNMPValue)
									{
										CurSNMPValue = Myatof(SnmpData->Data);
										SnmpData->Data = dfree(SnmpData->Data);
										
										// If Current value is bigger then the prev value we can subtract, otherwise assume that the
										// counter rolled.
										if(CurSNMPValue >= PrevSNMPValue)
										{
											SnmpData->Data = dsprintf(SnmpData->Data, "%.2f", (CurSNMPValue - PrevSNMPValue));
										} else {
											// [TCC] Ok, this is broken. We only support Counter32's.
											SnmpData->Data = dsprintf(SnmpData->Data, "%.2f", (Counter32Max - PrevSNMPValue) + CurSNMPValue);
										}
										
									} else {
										SnmpData->Data = dfree(SnmpData->Data);
										SnmpData->Data = dstrcpy(SnmpData->Data, "0");
									}
								}
							}
							
							// We Format the Data for display
							if(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.FORMAT"))
							{
								FORMAT = dstrcpy(FORMAT, ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.FORMAT"));
								FormatData = Check_Network_SNMP_FormatData(SnmpData->Data, FORMAT);
							} else {
								FORMAT = dstrcpy(FORMAT, "NONE");
								FormatData = dstrcpy(FormatData, SnmpData->Data);
							}
							
							if(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.LABEL"))
							{
								LABEL = dstrcpy(LABEL, ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.SNMP.LABEL"));
							} else {
								LABEL = dstrcpy(LABEL, OID);
							}
							
							if(IsStringNumeric(SnmpData->Data))
							{	
								tSTAT = dsprintf(tSTAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>%d</ORDER><LABEL>%s</LABEL><FORMAT>%s</FORMAT><DATAVAL>%s</DATAVAL></STAT>",
											ReturnXMLLeafValue(CheckXMLTree, "SERVER.RETURNCHECKS.CHECK.CID"),
											ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
				    							OptionCount,
				    							LABEL, FORMAT, 
				    							SnmpData->Data);
								STAT = dstrcat(STAT, tSTAT);
								tSTAT = dfree(tSTAT);
							}
							
							// Are we excepcted to Compare this value to something?
							if(LOGIC && EXPECT)
							{
								// Does it Check to TRUE or FASLE?
								if(!CheckStringLogic(SnmpData->Data, "", LOGIC, EXPECT))
								{
									Status = 0;
									tDetail = dsprintf(tDetail, "[RED] %s = %s (%s %s)\n", LABEL, FormatData, LOGIC, EXPECT); 
								} else {
									tDetail = dsprintf(tDetail, "[GREEN] %s = %s (%s %s)\n", LABEL, FormatData, LOGIC, EXPECT);
								}
							} else {
								// We just display The Data value.
								tDetail = dsprintf(tDetail, "[N/A] %s = %s\n", LABEL, FormatData);
							}
							
							Detail = dstrcat(Detail, tDetail);
							tDetail = dfree(tDetail);
						
							FormatData = dfree(FormatData);
							FORMAT = dfree(FORMAT);
							LABEL = dfree(LABEL);
						}
						
						if(SnmpData->Status == -1)
						{
							Status = 0;
							tDetail = dsprintf(tDetail, "[RED] OID:%s No Response from Device\n", OID);		
							Detail = dstrcat(Detail, tDetail);
							tDetail = dfree(tDetail);
						}
						
						if(SnmpData->Status == -2)
						{
							Status = 0;
							tDetail = dsprintf(tDetail, "[RED] OID:%s Connection to Device failed\n", OID);
							Detail = dstrcat(Detail, tDetail);
							tDetail = dfree(tDetail);
						}
					} else {
						Status = 0;
						tDetail = dsprintf(tDetail, "[RED] OID:%s Connection to Device failed\n", OID);
						Detail = dstrcat(Detail, tDetail);
						tDetail = dfree(tDetail);
					}
						                                                                                                        
					SnmpData->Data = dfree(SnmpData->Data);
					SnmpData = (struct SNMPData *) dfree((char *) SnmpData);
				} else {
					Status = 0;
                                        tDetail = dsprintf(tDetail, "[RED] Missing required Check Option values.\n");
					Detail = dstrcat(Detail, tDetail);
					tDetail = dfree(tDetail);
                                }
                                         
				LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
				CheckLeaf = DelSubXMLTree(CheckLeaf);
				OptionCount++;
			} else {
				if(OptionCount == 0)
				{
					Detail = dstrcat(Detail, "[RED] No SNMP check specified");
					Status = 0;
				}
			}
			
			if(LocalLeaf)
			{
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
	} else {
		Detail = dstrcpy(Detail, "No check to perform.");
	}
	
	if(!STAT)
	{
		STAT = dstrcpy(STAT, "");
	}
	
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL>%s</CHECK>", STATUS[Status], Detail, STAT);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);
	
	Detail = dfree(Detail);
	STAT = dfree(STAT);

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Network_SNMP]-[Exit]");
	
	return(Response);
}

/******************************************************************************
*	char * Check_Network_SNMP_FormatData(char * Data, char * FORMAT)
*		Returns a Data value Formated 
*	Args :
*		char * Data	String of Data Value.
*		char * FORMAT	Format to represent data as.
*
*       Updated :
******************************************************************************/
char * Check_Network_SNMP_FormatData(char * Data, char * FORMAT)
{
	char * Out = 0;
	char SUFFIX[][5] = { "KB", "MB", "GB" };
	float Value = 0;
	int Days = 0;
	int Hours = 0;
	int Minutes = 0;
	int Seconds = 0;
	float Divisor = 0;
	int Suffix = 0;
		
	if(Data && FORMAT)
	{
		if(!dstrcasecmp(FORMAT, "TIMETICKS"))
		{
			Value = Myatof(Data);
			if(Value)
			{
				Days = Value / 8544650;
				Value = Value - ((float) Days * 8544650);

				Hours = Value / (8544650 / 24);
				Value = Value - ((float) Hours * (8544650 / 24));

				Minutes = Value / (8544650 / 1440);
				Value = Value - ((float) Minutes * (8544650 / 1440));

				Seconds = Value / 100;
				
				Out = dsprintf(Out, "%d Days, %d Hours, %d Minutes, %d Seconds", Days, Hours, Minutes, Seconds);
			}
		}
		
		if(!dstrcasecmp(FORMAT, "BYTES"))
		{
			Value = Myatof(Data);
			
			if(Value >= 1024000000)
			{
				Divisor = 1024000000;
				Suffix = 2;
			} else if(Value >= 1024000)
			{
				Divisor = 1024000;
				Suffix = 1;
			} else if(Value >= 1024)
			{
				Divisor = 1024;
				Suffix = 0;
			}
			
			if(Divisor)
			{
				Out = dsprintf(Out, "%.2f%s", (Value / Divisor), SUFFIX[Suffix]);
			}
		}
	}
		
	if(!Out)
	{
		Out = dstrcpy(Out, Data);
	}

	return(Out);
}

/******************************************************************************
*       float Check_Network_SNMP_ReturnPrevResult(char * HID, char * CID, int OptionNum)
*		Returns the previous SNMP result stored in the local SNMP DB 
*		file (if any). 
*
*	Args :
*		char * HID	Host ID of host being tested.
*		char * CID	Check ID of SNMP check being performed.
*		int OptionNum	Option number for this check.
*
*       Updated :
******************************************************************************/
float Check_Network_SNMP_ReturnPrevResult(char * HID, char * CID, int OptionNum)
{
	FILE *fp;		
	struct SNMPDB SNMPValue; 
	char * SNMPFILE = 0;
	int LocalHID = 0;
	int LocalCID = 0;
	
	SNMPFILE = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DBFILE.SNMP");
	if(SNMPFILE)
	{
		LocalHID = Myatoi(HID);
		LocalCID = Myatoi(CID);
		
		fp = fopen(SNMPFILE,"r+b");
		if(fp)
		{
			do
			{
				fread(&SNMPValue, sizeof(struct SNMPDB), 1, fp);

				if(SNMPValue.HID == LocalHID && 
				   SNMPValue.CID == LocalCID && 
				   SNMPValue.OptionNum == OptionNum)
				{
					fclose(fp);
					return(SNMPValue.Value);
				}
			}while(!feof(fp));
			fclose(fp);
		}
	}
	return(0);
}

/******************************************************************************
*       void Check_Network_SNMP_UpdateDB(char * HID, char * CID, int OptionNum, char * Value)
*		Updates local SNMP DB with current SNMP result.
*
*	Args :
*		char * HID	Host ID of host being tested.
*		char * CID	Check ID of SNMP check being performed.
*		int OptionNum	Option number for this check.
*		char * Value	Value of SNMP result to be stored.
*
*       Updated :
******************************************************************************/
void Check_Network_SNMP_UpdateDB(char * HID, char * CID, int OptionNum, char * Value)
{
	FILE *fp;		
	struct SNMPDB SNMPRead;
	struct SNMPDB SNMPWrite;
	char * SNMPFILE = 0;
	int LocalHID = 0;
	int LocalCID = 0;
	float LocalValue = 0;
	int found = 0;
	long fpos = 0;
			
	SNMPFILE = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DBFILE.SNMP");
	if(SNMPFILE)
	{
		LocalHID = Myatoi(HID);
		LocalCID = Myatoi(CID);
		LocalValue = Myatof(Value);
		
		SNMPWrite.HID = LocalHID;
		SNMPWrite.CID = LocalCID;
		SNMPWrite.OptionNum = OptionNum;
		SNMPWrite.Value = LocalValue;
					
		fp = fopen(SNMPFILE,"r+b");
	       	if(!fp)
        	{
        		fp = fopen(SNMPFILE, "w+b");
        	}
 
		if(fp)
		{
			do
			{
				fpos = ftell(fp);
				fread(&SNMPRead, sizeof(struct SNMPDB), 1, fp);

				if(SNMPRead.HID == LocalHID && 
				   SNMPRead.CID == LocalCID && 
				   SNMPRead.OptionNum == OptionNum)
				{
					fseek(fp, fpos, SEEK_SET);
					fwrite(&SNMPWrite, sizeof(struct SNMPDB), 1, fp);
					found = 1;
					break;
				}
			}while(!feof(fp));
			
			if(!found)
			{
				fwrite(&SNMPWrite, sizeof(struct SNMPDB), 1, fp);
			}
			fclose(fp);
		}
	}
}
