/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASd
*       Filename : MIDASd.c
*		Main functions for MIDASd (Data Collection Daemon).
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*		12/16/2003 - Replaced FreeFork/CatchHUP with CatchSignal()
******************************************************************************/
#include <mysql/mysql.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <signal.h>
#include <wait.h>

#include "../Common/Commoninclude.h"

//
// MIDASd includes
#include "includes.h"

//
//	Global Var defined
//
char APPNAME[] = "MIDASd";
char * CFGPATH = 0;
char * DEBUGMSG = 0;
char * KEYFILE = 0;
char STATUS[4][10] = {
		"RED", 
		"GREEN", 
		"YELLOW"
		};

struct SQLCFG * SQLCFG;
struct XMLLeaf * ConfigXMLTree = 0;

int DEBUGLEVEL = 0;
int LOGTOSTDERR = 0;

//
// Setting Logging Bits;
//
int	LOG_INFO	=1;	//General Info
int	LOG_ERROR	=2;	//Errors
int	LOG_SQL		=4;	//SQL Statements
int	LOG_SOCKET	=8;	//Socket related
int	LOG_CHECK	=16;	//Checks
int	LOG_FORK	=32;	//Fork Processes
int	LOG_XML		=4096;	//XML
int	LOG_ALWAYS	=32767;

/******************************************************************************
*       int main(int argc, char *argv[])
*		main() function for MIDASd
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument string array.
*
*       Updated :
*		12/16/2003 - Changed how signal()'s are handled.
******************************************************************************/
int main(int argc, char *argv[])
{
	(void) signal(SIGHUP, CatchSignal);
	(void) signal(SIGCHLD, CatchSignal);
	(void) signal(SIGKILL, CatchSignal);
	(void) signal(SIGTERM, CatchSignal);
	
	CFGPATH = dsprintf(CFGPATH, "%s/etc/MIDASd.cf", APPROOT);
	ConfigXMLTree = FileToXMLTree(CFGPATH);
	if(!ConfigXMLTree)
	{
		printf("! Unable to open %s\n", CFGPATH);
		exit(0);
	}
	
	DEBUGLEVEL = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DEBUGLEVEL"));
	KEYFILE = dstrcpy(KEYFILE, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.KEYFILE"));
	
	if(!ParseCommandLine(argc, argv))
	{
		exit(0);
	}
	
	SQLCFG = (struct SQLCFG *) drealloc((char *) SQLCFG, sizeof(struct SQLCFG));
	
	SQLCFG->IP = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.IP");
	SQLCFG->user = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.USER");
	SQLCFG->passwd = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.PASSWD");
	SQLCFG->db = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.DB");
	SQLCFG->unixsocket = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.UNIXSOCKET");
	SQLCFG->port = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.port")); 
		
	
	if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
	{
		if(fork())
		{
			exit(0);
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[main]-[MIDASd %s starting]", MIDASVER);
	
	// Fork off our scheduled maint process
	if(!fork())
	{
		StartMaint();
		exit(0);
	}
	
	StartServer();
	
	DelXMLLeafTree(ConfigXMLTree);
	dfree((char *)SQLCFG);
	
	exit(0);

}

/******************************************************************************
*       void CatchSignal(int sig)
*		The signal() catch function. Handles, TERM, KILL, CHLD and HUP
*
*	Args :
*		int sig		Signal that caused trigger.
*
*       Updated :
******************************************************************************/
void CatchSignal(int sig)
{
	pid_t pid = 1;
	
	switch(sig)
	{
		case SIGHUP:
			MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchSignal]-[Caught HUP Reloading config file : %s]", CFGPATH);
			
			ConfigXMLTree = FileToXMLTree(CFGPATH);
			break;
		case SIGCHLD:
			while(pid > 0)
			{
				pid = waitpid(0, NULL, WNOHANG);
			}
			break;
		case SIGTERM:
		case SIGKILL:
			MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchHUP]-[Caught TERM/KILL exiting]");	
			exit(0);
			break;
	}
}

/******************************************************************************
*       int ParseCommandLine(int argc, char *argv[])
*		Parses command line arguments.
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument string array.
*
*       Updated :
******************************************************************************/
int ParseCommandLine(int argc, char *argv[])
{
	int opt = 0;
	int Status = 1;
	char *Key = 0;
	FILE *fp;
		
	while(opt != -1)
	{
		opt = getopt(argc, argv, "KVhTD:L");
		switch(opt)
		{
			case 'K':
				Key = MakeKey();
				fp = fopen(KEYFILE, "w");
				if(fp)
				{
					fprintf(fp, "%s", Key);
					printf("%s %s\n", APPNAME, MIDASVER);
					printf("New key generated in %s.\n", KEYFILE);
					fclose(fp);
				} else {
					printf(" ! Unable to open %s\n", KEYFILE);
				}
				Status = 0;
				break;
			case 'V':
				printf("%s %s\n", APPNAME, MIDASVER);
				Status = 0;
				break;
			case 'h':
				printf("%s %s\n", APPNAME, MIDASVER);
				printf("\t-D#\tSet Debug level\n");
				printf("\t-h\tShow (this) help screen\n");
				printf("\t-K\tBuild keyfile (%s)\n", KEYFILE);
				printf("\t-T\tShow Configuration\n");
				printf("\t-V\tShow version\n");
				printf("\t-L\tSend Debug to STDERR\n");
				
				Status = 0;
				break;
			case 'T':
				ShowConfig();
				Status = 0;
				break;
			case 'D':
				DEBUGLEVEL = Myatoi(optarg);
				break;
			case 'L':
				LOGTOSTDERR = 1;
				break;
		}
	}
	return(Status);
}

/******************************************************************************
*       void ShowConfig(void)
*		Will display the current configuration settings. This is 
*		triggered witht he -T command line option.
*
*
*       Updated :
******************************************************************************/
void ShowConfig()
{
        printf("%s %s\n", APPNAME, MIDASVER);
	printf("------------------------------------------------------------------\n");
        printf("[Contents of %s.cf file]\n", APPNAME); 
        DebugXMLTree("MIDASd.cf", ConfigXMLTree, 0);
}

