/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASn
*       Filename : stats.c
*		Funcions for handling stat gathering/reporting for MIDASn
*
*       Updated :
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"

/******************************************************************************
*       struct NetStat * UpdateStats(struct etherpacket * ep, struct NetStat * NetworkStat)
*		Determinds the type of packet we are looking at then increments the 
*		network stats.
*
*	Args :
*		struct etherpacket * ep		Ethernet packet.
*		struct NetStat * NetworkStat	Network stat struct.
*
*       Updated :
******************************************************************************/
struct NetStat * UpdateStats(struct etherpacket * ep, struct NetStat * NetworkStat)
{
	struct iphdr * ip;

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats]-[Enter]");
	
	ip = (struct iphdr *) (((unsigned long) &ep->ip) - 2);
	
	NetworkStat->TotalPkt += htons(ip->tot_len);
	NetworkStat->TotalPktCount++;
		
	switch(ip->protocol)
	{
		case 1:
			NetworkStat->ICMPPkt += htons(ip->tot_len);
			NetworkStat->ICMPPktCount++;
			break;
		case 6:
			NetworkStat->TCPPkt += htons(ip->tot_len);
			NetworkStat->TCPPktCount++;
			break;
		case 17:
			NetworkStat->UDPPkt += htons(ip->tot_len);
			NetworkStat->UDPPktCount++;
			break;
		default:
			NetworkStat->OtherPkt += htons(ip->tot_len);
			NetworkStat->OtherPktCount++;
			break;
	}
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats]-[Exit]");
	
	return(NetworkStat);
}


/******************************************************************************
*       void SendStats(struct NetStat * NetworkStat)
*		Builds NETSTAT query and sends to MIDASd.
*
*	Args :
*		struct NetStat * NetworkStat	The Network stats to use to 
*						build query with.
*
*       Updated :
******************************************************************************/			
void SendStats(struct NetStat * NetworkStat)
{
	char * Query = 0;
	char * tQuery = 0;
	char * Response = 0;
			
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[SendStats]-[Enter]");
	
	if(NetworkStat)
	{
		Query = dsprintf(Query, "<CLIENT><HID>%s</HID><TYPE>NETSTAT</TYPE><NETSTAT>", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"));
		
		// Total
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>0</ORDER><LABEL>Total Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->TotalPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>0</ORDER><LABEL>Total Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->TotalPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		// TCP
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>1</ORDER><LABEL>TCP Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->TCPPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>1</ORDER><LABEL>TCP Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->TCPPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
				
		// UDP
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>2</ORDER><LABEL>UDP Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->UDPPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>2</ORDER><LABEL>UDP Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->UDPPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
				
		// ICMP
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>3</ORDER><LABEL>ICMP Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->ICMPPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>3</ORDER><LABEL>ICMP Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->ICMPPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		// Other
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>4</ORDER><LABEL>Other Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->OtherPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>4</ORDER><LABEL>Other Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->OtherPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
				
		// Port Stats
		tQuery = UpdateStats_BuildPortsQuery(NetworkStat);
		if(tQuery)
		{
			Query = dstrcat(Query, tQuery);
		}
		tQuery = dfree(tQuery);
		
		// [TCC] Disabling Net Sessions untill v2.3		
		/*
		// Session Stats
		tQuery = UpdateStats_BuildSessionStats(NetworkStat);
		if(tQuery)
		{
			Query = dstrcat(Query, tQuery);
		}
		tQuery = dfree(tQuery);
		*/

		// Finish up the NETSTAT Segment
		Query = dstrcat(Query, "</NETSTAT>");
		
		
		// MAC/IP list
		tQuery = UpdateStats_BuildMacQuery(NetworkStat);
		if(tQuery)
		{
			Query = dstrcat(Query, tQuery);
		}
		tQuery = dfree(tQuery);

		// [TCC] Disabled until v2.3
		// Session list
		tQuery = UpdateStats_BuildSessionQuery(NetworkStat);
		if(tQuery)
		{
			Query = dstrcat(Query, tQuery);
		}
		tQuery = dfree(tQuery);
		
		// Finish up the Client Query
		Query = dstrcat(Query, "</CLIENT>");
		
		Response = SendToServer(Query);
		Response = dfree(Response);
		
		Query = dfree(Query);
	}
				
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[SendStats]-[Exit]");	
}
