/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASn
*       Filename : stats_ports.c
* 		Functions for gathering port stats
*
*       Updated :
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"

/******************************************************************************
*       struct NetStat * UpdateStats_Ports(struct etherpacket * ep, 
*					   struct NetStat * NetworkStat, 
*					   struct XMLLeaf * PortLeaf)
*		Updates the port statistics. Uses the PortLeaf to figure out if
*		its a port were wathcing for. If so then it updates the stats.
*
*	Args :
*		struct etherpacket * ep		Ethernet packet.
*		struct NetStat * NetworkStat	Network stats struct.
*		struct XMLLeaf * PortLeaf	XML LL contatining the ports to 
*						watch.
*
*       Updated :
******************************************************************************/
struct NetStat * UpdateStats_Ports(struct etherpacket * ep, struct NetStat * NetworkStats)
{
	struct iphdr * ip;
	struct tcphdr * tcp;
	struct udphdr * udp;
	struct icmphdr * icmp;
	unsigned SrcPort = 0;
	unsigned DstPort = 0;
	unsigned Len = 0;
	int Found = 0;
	char * Proto = 0;
	
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_Ports]-[Enter]");
	
	if(ep)
	{		
		ip = (struct iphdr *) (((unsigned long) &ep->ip) - 2);
	
		switch(ip->protocol)
		{
			case 1:
				icmp = (struct icmphdr *) (((unsigned long) &ep->tcp) - 2);
				break;
			case 6:
				tcp = (struct tcphdr *) (((unsigned long) &ep->tcp) - 2);
			
				SrcPort = htons(tcp->source);
				DstPort = htons(tcp->dest);
				Len = htons(ip->tot_len);
				
				Proto = dstrcpy(Proto, "TCP");
				Found = 1;
				break;
			case 17:
				udp = (struct udphdr *) (((unsigned long) &ep->tcp) - 2);
				
				SrcPort = htons(udp->source);
				DstPort = htons(udp->dest);
				Len = htons(ip->tot_len);
				
				Proto = dstrcpy(Proto, "UDP");
				Found = 1;
				break;
			default:
				break;
		}
		
		if(Found)
		{
			NetworkStats = IncrementPort(NetworkStats, Proto, SrcPort, DstPort, Len);
		}
		
		Proto = dfree(Proto);
		
	}
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_Ports]-[Exit]");
	
	return(NetworkStats);
}

/******************************************************************************
*       struct PortStat * UpdateStats_IncrementPort(struct PortStat * PortStats, char * Protocol, unsigned SrcPort, unsigned DstPort, unsigned Len)
*		Increment port stats with packet information.
*
*	Args :
*		struct PortStat * PortStats	Stored port statistics
*		char * Protocol			Packet porotocol.
*		unsigned SrcPort		Source port of packet.
*		unsigned DstPort		Destination port of packet.
*		unsigned Len			Length of packet.
*
*       Updated :
******************************************************************************/
struct NetStat * IncrementPort(struct NetStat * NetworkStats, char * Protocol, unsigned SrcPort, unsigned DstPort, unsigned Len)
{
	struct PortStat * LocalStats = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_IncrementPort]-[Proto:%s]-[SrcPort:%u]-[DstPort:%u]-[Len:%u]", Protocol, SrcPort, DstPort, Len);
	
	LocalStats = NetworkStats->PortStats;
	while(LocalStats)
	{
		if(LocalStats)
		{
			if(!dstrcasecmp(LocalStats->Protocol, Protocol))
			{
				if(LocalStats->SrcPort == SrcPort)
				{
					if(LocalStats->DstPort == DstPort || LocalStats->DstPort == 0)
					{
						LocalStats->Count++;
						LocalStats->Bytes=LocalStats->Bytes+Len;
						break;
					}
				}
				
				if(LocalStats->DstPort == DstPort)
				{
					if(LocalStats->SrcPort == SrcPort || LocalStats->SrcPort == 0)
					{
						LocalStats->Count++;
						LocalStats->Bytes=LocalStats->Bytes+Len;
						break;
					}
				}
			}
		
			LocalStats = LocalStats->NextStat;
		}
	}

	return(NetworkStats);
}

/******************************************************************************
*       struct PortStat * UpdateStats_PurgePortStats(struct PortStat * PortStats)
*		Clears/Frees Port statistics.
*
*	Args :
*		struct NetStat * NetworkStats	The NetStat struct to use.
*
*       Updated :
******************************************************************************/
struct NetStat * UpdateStats_PurgePortStats(struct NetStat * NetworkStats)
{
	struct PortStat * LocalStats = 0;
	struct PortStat * NextStats = 0;
	unsigned Count = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_PurgeStats]-[Enter]");
	
	if(NetworkStats)
	{
		LocalStats = NetworkStats->PortStats;
	
		if(LocalStats)
		{
			while(LocalStats->NextStat)
			{
				Count++;
				NextStats = LocalStats->NextStat;
				
				LocalStats->Protocol = dfree((char *) LocalStats->Protocol);
				LocalStats = (struct PortStat *) dfree((char *) LocalStats);
				
				LocalStats = NextStats;
			}
			
			if(LocalStats)
			{
				LocalStats->Protocol = dfree((char *) LocalStats->Protocol);
				LocalStats = (struct PortStat *) dfree((char*) LocalStats);
			}
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_PurgeStats]-[Exit]-[Elements:%u]", Count);

	return(NetworkStats);
}

/******************************************************************************
*	struct NetStat * UpdateStats_InitPorts(struct NetStat * NetworkStats, struct XMLLeaf * PortLeaf)
*		Init's the Port stats Linked list to starting values.
*
*	Args :
*		struct NetStat * NetworkStats	The NetStat struct to use.
*		struct XMLLeaf * PortLeaf	The XML Port watch list
*
*       Updated :
******************************************************************************/
struct NetStat * UpdateStats_InitPorts(struct NetStat * NetworkStats, struct XMLLeaf * PortLeaf)
{
	struct XMLLeaf * LocalLeaf = 0;
	struct XMLLeaf * SubLeaf = 0;
	struct PortStat * NewStat = 0;
	struct PortStat * RootStat = 0;
	struct PortStat * PrevStat = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_InitPorts]-[Enter]");
	
	if(PortLeaf)
	{
		LocalLeaf = PortLeaf;
		SubLeaf = PortLeaf;
		while(LocalLeaf)
		{
			SubLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.NETMON.PORT"), 1);
			
			if(SubLeaf)
			{
				if(ReturnXMLLeafValue(SubLeaf, "SERVER.NETMON.PORT.PROTO"))
				{
					if(NewStat)
					{
						PrevStat = NewStat;
					}
					
					NewStat = 0;
					NewStat = (struct PortStat *) drealloc((char *)NewStat, sizeof(struct PortStat));
					NewStat->Protocol = dstrcpy(NewStat->Protocol, ReturnXMLLeafValue(SubLeaf, "SERVER.NETMON.PORT.PROTO"));
					NewStat->SrcPort = Myatoll(ReturnXMLLeafValue(SubLeaf, "SERVER.NETMON.PORT.SRC"));
					NewStat->DstPort = Myatoll(ReturnXMLLeafValue(SubLeaf, "SERVER.NETMON.PORT.DST"));
				
					if(PrevStat)
					{
						PrevStat->NextStat = NewStat;
					} else {
						RootStat = NewStat;
					}
				}
				LocalLeaf = FindXMLLeafByID(LocalLeaf, SubLeaf->ID);
				SubLeaf = DelSubXMLTree(SubLeaf);
			}
		
			if(LocalLeaf)
			{
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
		NetworkStats->PortStats = RootStat;
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_InitPorts]-[Exit]");
	
	return(NetworkStats);
}

char * UpdateStats_BuildPortsQuery(struct NetStat * NetworkStats)
{
	struct PortStat * PortStats = 0;
	char * Query = 0;
	char * tQuery = 0;
	unsigned PortCount = 0;
	unsigned Port = 0;
	
	if(NetworkStats->PortStats)
	{
		PortStats = NetworkStats->PortStats;
			
		while(PortStats)
		{
			if(PortStats)
			{
				if(PortStats->SrcPort)
				{
					Port = PortStats->SrcPort;
				} else {
					Port = PortStats->DstPort;
				}
									
				tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>PORT_COUNT_%s_%u</ID><ORDER>%u</ORDER><LABEL>%s Port %u (Packets)</LABEL><DATAVAL>%u</DATAVAL></STAT>",
							  PortStats->Protocol,
							  Port,
							  PortCount,
							  PortStats->Protocol,
							  Port,
							  PortStats->Count);
				Query = dstrcat(Query, tQuery);
				tQuery = dfree(tQuery);
					
				tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>PORT_BYTES_%s_%u</ID><ORDER>%u</ORDER><LABEL>%s Port %u (Bytes)</LABEL><DATAVAL>%u</DATAVAL></STAT>",
							  PortStats->Protocol,
							  Port,
							  PortCount, 
							  PortStats->Protocol,
							  Port,
							  PortStats->Bytes);
				Query = dstrcat(Query, tQuery);
				tQuery = dfree(tQuery);
	
				PortStats = PortStats->NextStat;
				PortCount++;
			}
		}
	}

	return(Query);
}
