/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASs
*       Filename : MIDASs.c
* 		Handles the interfacing of Snort (http://www.snort.org) and 
*		MIDASd. Basic operation : Takes piped output of Snort, stores
*		IP information localy, send off alerts to MIDASd
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"


struct XMLLeaf * ConfigXMLTree = 0;
char * CFGPATH = 0;
char APPNAME[] = "MIDASs";

char * KEYFILE = 0;

int LOGTOSTDERR = 0;
int DEBUGLEVEL 	= 0;
int LOG_INFO 	= 2;
int LOG_SNORT 	= 4;
int LOG_SOCKET 	= 8;
int LOG_CHECK 	= 16;
int LOG_XML	= 32;
int LOG_FORK	= 64;

/******************************************************************************
*       int main(int argc, char *argv[])
*		main() function for MIDASs
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument string array.
*
*       Updated :
******************************************************************************/
int main(int argc, char *argv[])
{
	int Status = 0;
	
	(void) signal(SIGHUP, CatchSignal);
	(void) signal(SIGKILL, CatchSignal);
	(void) signal(SIGTERM, CatchSignal);
	
	CFGPATH = dsprintf(CFGPATH, "%s/etc/MIDASs.cf", APPROOT);
	ConfigXMLTree = FileToXMLTree(CFGPATH);
		
	if(!ConfigXMLTree)
	{
		printf("! Unable to read %s config file.\n", CFGPATH);
		exit(0);
	}

	DEBUGLEVEL = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DEBUGLEVEL"));

	KEYFILE = dstrcpy(KEYFILE, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.KEYFILE"));
	
	if(!ParseCommandLine(argc, argv))
	{
		exit(0);
	}
	
	if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
	{
		if(fork())
		{
			exit(0);
		}
	}
			
	do
	{
		if(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SNORT.CMD"))
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[Main]-[SNORT Starting]");
			
			Status = Snort_Start();
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[Main]-[SNORT Ending]");
			
			if(Status)
			{
				// Snort config worked and snort executed, but bombed. We pause 
				// 30 seconds and then start over.
				sleep(30);
			} else {
				// Couldnt Execute Snort exiting application
				exit(-1);
			}
		}
				
		if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
		{
			sleep(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.LOOPTIME")));
		}
		
	}while(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")));
	
	exit(0);	
}

/******************************************************************************
*       int ParseCommandLine(int argc, char *argv[])
*		Parses command line arguments.
*
*	Args :
*		int args	Argument count.
*		char *argc[]	Argument string array.
*
*       Updated :
******************************************************************************/
int ParseCommandLine(int argc, char *argv[])
{
	int opt = 0;
	int Status = 1;
		
	while(opt != -1)
	{
		opt = getopt(argc, argv, "VhTD:L");
		switch(opt)
		{
			case 'V':
				printf("%s %s\n", APPNAME, MIDASVER);
				Status = 0;
				break;
			case 'h':
				printf("%s %s\n", APPNAME, MIDASVER);
				printf("\t-V\tShow version\n");
				printf("\t-h\tShow (this) help screen\n");
				printf("\t-D\tSet Debuging level\n");
				printf("\t-L\tSend Debug to STDERR\n");
				
				Status = 0;
				break;
			case 'T':
				ShowConfig();
				Status = 0;
				break;
			case 'D':
				DEBUGLEVEL = Myatoi(optarg);
				break;
			case 'L':
				LOGTOSTDERR = 1;
				break;
		}
	}
	return(Status);
}

/******************************************************************************
*       void ShowConfig(void)
*		Shows current configuration setting. Triggered with -T command
*		line switch.
*
*
*       Updated :
******************************************************************************/
void ShowConfig(void)
{
        printf("%s %s\n", APPNAME, MIDASVER);
        printf("------------------------------------------------------------------\n");
        printf("[Contents of %s.cf file]\n", APPNAME);
        DebugXMLTree("MIDASa.cf", ConfigXMLTree, 0);
}

/******************************************************************************
*       void CatchSignal(int sig)
*               The signal() catch function. Handles, TERM, KILL, CHLD and HUP
*
*       Args :
*               int sig         Signal that caused trigger.
*
*       Updated :
******************************************************************************/
void CatchSignal(int sig)
{
        switch(sig)
        {
                case SIGHUP:
                        MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchSignal]-[Caught HUP Reloading config file : %s]", CFGPATH);

                        ConfigXMLTree = FileToXMLTree(CFGPATH);
                        break;
                case SIGTERM:
                case SIGKILL:
                        MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchHUP]-[Caught TERM/KILL exiting]");
                        exit(0);
                        break;
        }
}
